/**
 * Copyright 2013-2015 Christian Pierre MOMON, DEVINSY, UMR 7186 LESC.
 * 
 * christian.momon@devinsy.fr
 * 
 * This file is part of Kiwa. This software (Kiwa) is a computer program whose
 * purpose is to be the Kinsources Web Application, an open interactive platform
 * for archiving, sharing, analyzing and comparing kinship data used in
 * scientific inquiry.
 * 
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/ or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
package org.kinsources.kiwa.actilog;

import java.io.PrintWriter;

import org.apache.commons.lang3.StringEscapeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class represents a Actilog File reader and writer.
 * 
 * @author TIP
 */
public class CSVActilog
{
	private static final Logger logger = LoggerFactory.getLogger(CSVActilog.class);
	public static final char SEPARATOR = '\t';

	/**
	 * 
	 * @param source
	 * @return
	 */
	public static void write(final PrintWriter out, final Log source)
	{
		if (out == null)
		{
			throw new IllegalArgumentException("out is null.");
		}
		else
		{
			out.print(source.getId());
			out.print(SEPARATOR);
			out.print(source.getTime().toString());
			out.print(SEPARATOR);
			out.print(source.getLevel().toString());
			out.print(SEPARATOR);
			out.print(StringEscapeUtils.escapeXml(source.getEvent()));
			out.print(SEPARATOR);
			String details;
			if (source.getDetails() == null)
			{
				details = "";
			}
			else
			{
				details = StringEscapeUtils.escapeXml(source.getDetails());
			}
			out.print(details);
			out.println();
		}
	}

	/**
	 * 
	 * @param source
	 * @return
	 */
	public static void write(final PrintWriter out, final Logs source)
	{

		if (out == null)
		{
			throw new IllegalArgumentException("out is null.");
		}
		else if (source != null)
		{
			//
			out.print("id");
			out.print(SEPARATOR);
			out.print("time");
			out.print(SEPARATOR);
			out.print("level");
			out.print(SEPARATOR);
			out.print("event");
			out.print(SEPARATOR);
			out.print("details");
			out.println();

			//
			for (Log log : source)
			{
				write(out, log);
			}
		}
	}
}
